@@ -1,6 +1,6 @@
 /*
-  Copyright(C) 2009-2018 Brazil
-  Copyright(C) 2018-2019 Kouhei Sutou <kou@clear-code.com>
+  Copyright(C) 2009-2018  Brazil
+  Copyright(C) 2018-2020  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -130,7 +130,8 @@ typedef enum {
   GRN_SCORER_ERROR = -76,
   GRN_CANCEL = -77,
   GRN_WINDOW_FUNCTION_ERROR = -78,
-  GRN_ZSTD_ERROR = -79
+  GRN_ZSTD_ERROR = -79,
+  GRN_CONNECTION_RESET = -80
 } grn_rc;
 
 GRN_API grn_rc grn_init(void);
@@ -266,6 +267,9 @@ GRN_API grn_rc grn_set_default_encoding(grn_encoding encoding);
   ((ctx)->encoding = (enc == GRN_ENC_DEFAULT) ? grn_get_default_encoding() : enc)
 
 GRN_API const char *grn_get_version(void);
+GRN_API uint32_t grn_get_version_major(void);
+GRN_API uint32_t grn_get_version_minor(void);
+GRN_API uint32_t grn_get_version_micro(void);
 GRN_API const char *grn_get_package(void);
 GRN_API const char *grn_get_package_label(void);
 
@@ -363,6 +367,7 @@ typedef uint32_t grn_column_flags;
 #define GRN_OBJ_INDEX_SMALL            (0x01<<16)
 #define GRN_OBJ_INDEX_MEDIUM           (0x01<<17)
 #define GRN_OBJ_INDEX_LARGE            (0x01<<18)
+#define GRN_OBJ_WEIGHT_FLOAT32         (0x01<<19)
 
 /* flags only for grn_table_flags and grn_column_flags */
 
@@ -404,22 +409,19 @@ typedef uint32_t grn_column_flags;
 #define GRN_COLUMN_VAR_SIZE            (0x41)
 #define GRN_COLUMN_INDEX               (0x48)
 
-typedef struct _grn_section grn_section;
-typedef struct _grn_obj_header grn_obj_header;
-
-struct _grn_section {
-  unsigned int offset;
-  unsigned int length;
-  unsigned int weight;
+typedef struct _grn_section {
+  uint32_t offset;
+  uint32_t length;
+  float weight;
   grn_id domain;
-};
+} grn_section;
 
-struct _grn_obj_header {
-  unsigned char type;
-  unsigned char impl_flags;
+typedef struct _grn_obj_header {
+  uint8_t type;
+  uint8_t impl_flags;
   grn_obj_flags flags;
   grn_id domain;
-};
+} grn_obj_header;
 
 struct _grn_obj {
   grn_obj_header header;
@@ -432,7 +434,7 @@ struct _grn_obj {
     struct {
       grn_obj *body;
       grn_section *sections;
-      int n_sections;
+      uint32_t n_sections;
     } v;
   } u;
 };
@@ -484,7 +486,8 @@ typedef enum {
   GRN_DB_TEXT,
   GRN_DB_LONG_TEXT,
   GRN_DB_TOKYO_GEO_POINT,
-  GRN_DB_WGS84_GEO_POINT
+  GRN_DB_WGS84_GEO_POINT,
+  GRN_DB_FLOAT32
 } grn_builtin_type;
 
 typedef enum {
@@ -515,37 +518,8 @@ typedef struct {
 
 typedef grn_rc (*grn_plugin_func)(grn_ctx *ctx);
 
-typedef enum {
-  GRN_PROC_INVALID = 0,
-  GRN_PROC_TOKENIZER,
-  GRN_PROC_COMMAND,
-  GRN_PROC_FUNCTION,
-  GRN_PROC_HOOK,
-  GRN_PROC_NORMALIZER,
-  GRN_PROC_TOKEN_FILTER,
-  GRN_PROC_SCORER,
-  GRN_PROC_WINDOW_FUNCTION
-} grn_proc_type;
-
-GRN_API grn_obj *grn_proc_create(grn_ctx *ctx,
-                                 const char *name, int name_size, grn_proc_type type,
-                                 grn_proc_func *init, grn_proc_func *next, grn_proc_func *fin,
-                                 unsigned int nvars, grn_expr_var *vars);
-GRN_API grn_obj *grn_proc_get_info(grn_ctx *ctx, grn_user_data *user_data,
-                                   grn_expr_var **vars, unsigned int *nvars, grn_obj **caller);
-GRN_API grn_proc_type grn_proc_get_type(grn_ctx *ctx, grn_obj *proc);
-
 typedef grn_obj grn_table_cursor;
 
-typedef struct {
-  grn_id rid;
-  uint32_t sid;
-  uint32_t pos;
-  uint32_t tf;
-  uint32_t weight;
-  uint32_t rest;
-} grn_posting;
-
 typedef enum {
   GRN_OP_PUSH = 0,
   GRN_OP_POP,
@@ -628,7 +602,9 @@ typedef enum {
   GRN_OP_GET_MEMBER,
   GRN_OP_REGEXP,
   GRN_OP_FUZZY,
-  GRN_OP_QUORUM
+  GRN_OP_QUORUM,
+  GRN_OP_NEAR_PHRASE,
+  GRN_OP_ORDERED_NEAR_PHRASE,
 } grn_operator;
 
 GRN_API grn_obj *grn_obj_column(grn_ctx *ctx, grn_obj *table,
@@ -654,8 +630,11 @@ GRN_API grn_obj *grn_obj_column(grn_ctx *ctx, grn_obj *table,
 #define GRN_COLUMN_NAME_MIN_LEN       (sizeof(GRN_COLUMN_NAME_MIN) - 1)
 #define GRN_COLUMN_NAME_SUM           "_sum"
 #define GRN_COLUMN_NAME_SUM_LEN       (sizeof(GRN_COLUMN_NAME_SUM) - 1)
+/* Deprecated since 10.0.4. Use GRN_COLUMN_NAME_MEAN instead. */
 #define GRN_COLUMN_NAME_AVG           "_avg"
 #define GRN_COLUMN_NAME_AVG_LEN       (sizeof(GRN_COLUMN_NAME_AVG) - 1)
+#define GRN_COLUMN_NAME_MEAN          "_mean"
+#define GRN_COLUMN_NAME_MEAN_LEN      (sizeof(GRN_COLUMN_NAME_MEAN) - 1)
 
 GRN_API grn_obj *grn_column_create(grn_ctx *ctx, grn_obj *table,
                                    const char *name, unsigned int name_size,
@@ -764,7 +743,41 @@ GRN_API grn_rc grn_column_rename(grn_ctx *ctx, grn_obj *column,
 
 GRN_API grn_rc grn_obj_close(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_rc grn_obj_reinit(grn_ctx *ctx, grn_obj *obj, grn_id domain, unsigned char flags);
+/* On non reference count mode (default):
+ * This closes the following objects immediately:
+ *
+ *   * Acceessor
+ *   * Bulk
+ *   * DB
+ *   * Temporary column
+ *   * Temporary table
+ *
+ * This does nothing for other objects such as persisted tables and
+ * columns.
+ *
+ * On reference count mode (GRN_ENABLE_REFERENCE_COUNT=yes):
+ * This closes the following objects immediately:
+ *
+ *   * Bulk
+ *   * DB
+ *
+ * This decreases the reference count of the following objects:
+ *
+ *   * Acceessor
+ *   * Column (both persisted and temporary)
+ *   * Table (both persisted and temporary)
+ *
+ * If the decreased reference count is zero, the object is closed.
+ */
 GRN_API void grn_obj_unlink(grn_ctx *ctx, grn_obj *obj);
+GRN_API grn_rc grn_obj_refer(grn_ctx *ctx, grn_obj *obj);
+GRN_API grn_rc grn_obj_refer_recursive(grn_ctx *ctx, grn_obj *obj);
+GRN_API grn_rc grn_obj_refer_recursive_dependent(grn_ctx *ctx, grn_obj *obj);
+/* This calls grn_obj_unlink() only on reference count mode
+ * (GRN_ENABLE_REFERENCE_COUNT=yes) */
+GRN_API void grn_obj_unref(grn_ctx *ctx, grn_obj *obj);
+GRN_API void grn_obj_unref_recursive(grn_ctx *ctx, grn_obj *obj);
+GRN_API void grn_obj_unref_recursive_dependent(grn_ctx *ctx, grn_obj *obj);
 
 GRN_API grn_user_data *grn_obj_user_data(grn_ctx *ctx, grn_obj *obj);
 
@@ -833,42 +846,17 @@ struct _grn_search_optarg {
   grn_fuzzy_search_optarg fuzzy;
   grn_match_info match_info;
   int quorum_threshold;
+  int additional_last_interval;
 };
 
 GRN_API grn_rc grn_obj_search(grn_ctx *ctx, grn_obj *obj, grn_obj *query,
                               grn_obj *res, grn_operator op, grn_search_optarg *optarg);
 
-typedef grn_rc grn_selector_func(grn_ctx *ctx, grn_obj *table, grn_obj *index,
-                                 int nargs, grn_obj **args,
-                                 grn_obj *res, grn_operator op);
-
-GRN_API grn_rc grn_proc_set_selector(grn_ctx *ctx, grn_obj *proc,
-                                     grn_selector_func selector);
-GRN_API grn_rc grn_proc_set_selector_operator(grn_ctx *ctx,
-                                              grn_obj *proc,
-                                              grn_operator selector_op);
-GRN_API grn_operator grn_proc_get_selector_operator(grn_ctx *ctx,
-                                                    grn_obj *proc);
-
 GRN_API grn_rc grn_proc_set_is_stable(grn_ctx *ctx,
                                       grn_obj *proc,
                                       grn_bool is_stable);
 GRN_API grn_bool grn_proc_is_stable(grn_ctx *ctx, grn_obj *proc);
 
-/*-------------------------------------------------------------
- * grn_uvector
-*/
-
-GRN_API unsigned int grn_uvector_size(grn_ctx *ctx, grn_obj *uvector);
-GRN_API unsigned int grn_uvector_element_size(grn_ctx *ctx, grn_obj *uvector);
-
-GRN_API grn_rc grn_uvector_add_element(grn_ctx *ctx, grn_obj *vector,
-                                       grn_id id, unsigned int weight);
-
-GRN_API grn_id grn_uvector_get_element(grn_ctx *ctx, grn_obj *uvector,
-                                       unsigned int offset,
-                                       unsigned int *weight);
-
 /*-------------------------------------------------------------
  * API for hook
  */
@@ -922,37 +910,6 @@ GRN_API grn_rc grn_column_get_all_index_columns(grn_ctx *ctx,
 GRN_API grn_rc grn_obj_delete_by_id(grn_ctx *ctx, grn_obj *db, grn_id id, grn_bool removep);
 GRN_API grn_rc grn_obj_path_by_id(grn_ctx *ctx, grn_obj *db, grn_id id, char *buffer);
 
-/* geo */
-
-typedef struct {
-  int latitude;
-  int longitude;
-} grn_geo_point;
-
-GRN_API grn_rc grn_geo_select_in_rectangle(grn_ctx *ctx,
-                                           grn_obj *index,
-                                           grn_obj *top_left_point,
-                                           grn_obj *bottom_right_point,
-                                           grn_obj *res,
-                                           grn_operator op);
-GRN_API unsigned int grn_geo_estimate_size_in_rectangle(grn_ctx *ctx,
-                                                        grn_obj *index,
-                                                        grn_obj *top_left_point,
-                                                        grn_obj *bottom_right_point);
-/* Deprecated since 4.0.8. Use grn_geo_estimate_size_in_rectangle() instead. */
-GRN_API int grn_geo_estimate_in_rectangle(grn_ctx *ctx,
-                                          grn_obj *index,
-                                          grn_obj *top_left_point,
-                                          grn_obj *bottom_right_point);
-GRN_API grn_obj *grn_geo_cursor_open_in_rectangle(grn_ctx *ctx,
-                                                  grn_obj *index,
-                                                  grn_obj *top_left_point,
-                                                  grn_obj *bottom_right_point,
-                                                  int offset,
-                                                  int limit);
-GRN_API grn_posting *grn_geo_cursor_next(grn_ctx *ctx, grn_obj *cursor);
-
-
 /* query & snippet */
 
 #ifndef GRN_QUERY_AND
@@ -1212,6 +1169,13 @@ GRN_API off_t grn_default_query_logger_get_rotate_threshold_size(void);
     }\
   }\
 } while (0)
+#define GRN_BULK_SET_CURR(buf,p) do {\
+  if (GRN_BULK_OUTP(buf)) {\
+    (buf)->u.b.curr = (char *)(p);\
+  } else {\
+    (buf)->header.flags = (char *)(p) - GRN_BULK_HEAD(buf);\
+  }\
+} while (0)
 #define GRN_BULK_INCR_LEN(bulk,len) do {\
   if (GRN_BULK_OUTP(bulk)) {\
     (bulk)->u.b.curr += (len);\
@@ -1265,6 +1229,7 @@ GRN_API grn_rc grn_bulk_fin(grn_ctx *ctx, grn_obj *bulk);
 GRN_API grn_rc grn_text_itoa(grn_ctx *ctx, grn_obj *bulk, int i);
 GRN_API grn_rc grn_text_itoa_padded(grn_ctx *ctx, grn_obj *bulk, int i, char ch, size_t len);
 GRN_API grn_rc grn_text_lltoa(grn_ctx *ctx, grn_obj *bulk, long long int i);
+GRN_API grn_rc grn_text_f32toa(grn_ctx *ctx, grn_obj *bulk, float f);
 GRN_API grn_rc grn_text_ftoa(grn_ctx *ctx, grn_obj *bulk, double d);
 GRN_API grn_rc grn_text_itoh(grn_ctx *ctx, grn_obj *bulk, unsigned int i, size_t len);
 GRN_API grn_rc grn_text_itob(grn_ctx *ctx, grn_obj *bulk, grn_id id);
@@ -1280,8 +1245,15 @@ GRN_API grn_rc grn_text_escape_xml(grn_ctx *ctx, grn_obj *buf,
 GRN_API grn_rc grn_text_time2rfc1123(grn_ctx *ctx, grn_obj *bulk, int sec);
 GRN_API grn_rc grn_text_printf(grn_ctx *ctx, grn_obj *bulk,
                                const char *format, ...) GRN_ATTRIBUTE_PRINTF(3);
-GRN_API grn_rc grn_text_vprintf(grn_ctx *ctx, grn_obj *bulk,
-                                const char *format, va_list args);
+GRN_API grn_rc grn_text_printfv(grn_ctx *ctx,
+                                grn_obj *bulk,
+                                const char *format,
+                                va_list args);
+/* Deprecated since 10.0.3. Use grn_text_printfv() instead. */
+GRN_API grn_rc grn_text_vprintf(grn_ctx *ctx,
+                                grn_obj *bulk,
+                                const char *format,
+                                va_list args);
 
 typedef void (*grn_recv_handler_func)(grn_ctx *ctx, int flags, void *user_data);
 
@@ -1354,6 +1326,8 @@ GRN_API void grn_ctx_recv_handler_set(grn_ctx *,
   GRN_VALUE_FIX_SIZE_INIT(obj, flags, GRN_DB_INT64)
 #define GRN_UINT64_INIT(obj,flags) \
   GRN_VALUE_FIX_SIZE_INIT(obj, flags, GRN_DB_UINT64)
+#define GRN_FLOAT32_INIT(obj,flags) \
+  GRN_VALUE_FIX_SIZE_INIT(obj, flags, GRN_DB_FLOAT32)
 #define GRN_FLOAT_INIT(obj,flags) \
   GRN_VALUE_FIX_SIZE_INIT(obj, flags, GRN_DB_FLOAT)
 #define GRN_TIME_INIT(obj,flags) \
@@ -1404,6 +1378,10 @@ GRN_API void grn_ctx_recv_handler_set(grn_ctx *,
   uint64_t _val = (uint64_t)(val);\
   grn_bulk_write_from((ctx), (obj), (char *)&_val, 0, sizeof(uint64_t));\
 } while (0)
+#define GRN_FLOAT32_SET(ctx,obj,val) do {\
+  float _val = (float)(val);\
+  grn_bulk_write_from((ctx), (obj), (char *)&_val, 0, sizeof(float));\
+} while (0)
 #define GRN_FLOAT_SET(ctx,obj,val) do {\
   double _val = (double)(val);\
   grn_bulk_write_from((ctx), (obj), (char *)&_val, 0, sizeof(double));\
@@ -1476,6 +1454,11 @@ GRN_API void grn_ctx_recv_handler_set(grn_ctx *,
                       (offset) * sizeof(uint64_t),\
                       sizeof(uint64_t));\
 } while (0)
+#define GRN_FLOAT32_SET_AT(ctx,obj,offset,val) do {\
+  float _val = (float)(val);\
+  grn_bulk_write_from((ctx), (obj), (char *)&_val,\
+                      (offset) * sizeof(float), sizeof(float));\
+} while (0)
 #define GRN_FLOAT_SET_AT(ctx,obj,offset,val) do {\
   double _val = (double)(val);\
   grn_bulk_write_from((ctx), (obj), (char *)&_val,\
@@ -1502,6 +1485,7 @@ GRN_API void grn_ctx_recv_handler_set(grn_ctx *,
 #define GRN_UINT32_VALUE(obj) (*((uint32_t *)GRN_BULK_HEAD(obj)))
 #define GRN_INT64_VALUE(obj) (*((int64_t *)GRN_BULK_HEAD(obj)))
 #define GRN_UINT64_VALUE(obj) (*((uint64_t *)GRN_BULK_HEAD(obj)))
+#define GRN_FLOAT32_VALUE(obj) (*((float *)GRN_BULK_HEAD(obj)))
 #define GRN_FLOAT_VALUE(obj) (*((double *)GRN_BULK_HEAD(obj)))
 #define GRN_TIME_VALUE GRN_INT64_VALUE
 #define GRN_RECORD_VALUE(obj) (*((grn_id *)GRN_BULK_HEAD(obj)))
@@ -1521,6 +1505,7 @@ GRN_API void grn_ctx_recv_handler_set(grn_ctx *,
 #define GRN_UINT32_VALUE_AT(obj,offset) (((uint32_t *)GRN_BULK_HEAD(obj))[offset])
 #define GRN_INT64_VALUE_AT(obj,offset) (((int64_t *)GRN_BULK_HEAD(obj))[offset])
 #define GRN_UINT64_VALUE_AT(obj,offset) (((uint64_t *)GRN_BULK_HEAD(obj))[offset])
+#define GRN_FLOAT32_VALUE_AT(obj,offset) (((float *)GRN_BULK_HEAD(obj))[offset])
 #define GRN_FLOAT_VALUE_AT(obj,offset) (((double *)GRN_BULK_HEAD(obj))[offset])
 #define GRN_TIME_VALUE_AT GRN_INT64_VALUE_AT
 #define GRN_RECORD_VALUE_AT(obj,offset) (((grn_id *)GRN_BULK_HEAD(obj))[offset])
@@ -1559,6 +1544,9 @@ GRN_API void grn_ctx_recv_handler_set(grn_ctx *,
   uint64_t _val = (uint64_t)(val);\
   grn_bulk_write((ctx), (obj), (char *)&_val, sizeof(uint64_t));\
 } while (0)
+#define GRN_FLOAT32_PUT(ctx,obj,val) do {\
+  float _val = (float)(val); grn_bulk_write((ctx), (obj), (char *)&_val, sizeof(float));\
+} while (0)
 #define GRN_FLOAT_PUT(ctx,obj,val) do {\
   double _val = (double)(val); grn_bulk_write((ctx), (obj), (char *)&_val, sizeof(double));\
 } while (0)
@@ -1588,6 +1576,7 @@ GRN_API void grn_ctx_recv_handler_set(grn_ctx *,
 #define GRN_UINT32_POP(obj, value) GRN_BULK_POP(obj, value, uint32_t, 0)
 #define GRN_INT64_POP(obj, value) GRN_BULK_POP(obj, value, int64_t, 0)
 #define GRN_UINT64_POP(obj, value) GRN_BULK_POP(obj, value, uint64_t, 0)
+#define GRN_FLOAT32_POP(obj, value) GRN_BULK_POP(obj, value, float, 0.0)
 #define GRN_FLOAT_POP(obj, value) GRN_BULK_POP(obj, value, double, 0.0)
 #define GRN_TIME_POP GRN_INT64_POP
 #define GRN_RECORD_POP(obj, value) GRN_BULK_POP(obj, value, grn_id, GRN_ID_NIL)
@@ -1603,6 +1592,7 @@ GRN_API void grn_ctx_recv_handler_set(grn_ctx *,
 #define GRN_UINT32_VECTOR_SIZE(obj) GRN_BULK_VECTOR_SIZE(obj, uint32_t)
 #define GRN_INT64_VECTOR_SIZE(obj) GRN_BULK_VECTOR_SIZE(obj, int64_t)
 #define GRN_UINT64_VECTOR_SIZE(obj) GRN_BULK_VECTOR_SIZE(obj, uint64_t)
+#define GRN_FLOAT32_VECTOR_SIZE(obj) GRN_BULK_VECTOR_SIZE(obj, float)
 #define GRN_FLOAT_VECTOR_SIZE(obj) GRN_BULK_VECTOR_SIZE(obj, double)
 #define GRN_TIME_VECTOR_SIZE GRN_INT64_VECTOR_SIZE
 #define GRN_RECORD_VECTOR_SIZE(obj) GRN_BULK_VECTOR_SIZE(obj, grn_id)
