@@ -9,12 +9,15 @@ module Groonga
       @record_exist = record_exist
       @variable = @expression[0]
       @table = Context.instance[@variable.domain_id]
+      @expression.take_object(@table)
     end
 
     RELATION_OPERATORS = [
       Operator::MATCH,
       Operator::NEAR,
       Operator::NEAR2,
+      Operator::NEAR_PHRASE,
+      Operator::ORDERED_NEAR_PHRASE,
       Operator::SIMILAR,
       Operator::PREFIX,
       Operator::SUFFIX,
